@@ -407,8 +407,11 @@ class Pan(Transform):
         audio.from_mono(len(dBs))
         
         for (i, dB) in enumerate(dBs):
-            audio.audio[i,:len(dB)] *= DB_to_Linear(dB)
-            audio.audio[i,len(dB):] *= DB_to_Linear(self.scheme(self.pan.endpoint())[i])
+            if isnumber(dB):
+                audio.audio[i,:] *= DB_to_Linear(dB)
+            else: # paramterization
+                audio.audio[i,:len(dB)] *= DB_to_Linear(dB)
+                audio.audio[i,len(dB):] *= DB_to_Linear(self.scheme(self.pan.endpoint())[i])
 
 class Repan(Transform):
     """ Allows switching between channels.
@@ -478,7 +481,7 @@ class Downsample(Transform):
             raise NotImplementedError # TODO
     
     def realise(self, audio):
-        l = audio.audio.shape[1] #- self.phase
+        l = audio.length #- self.phase
         
         for i in range(1, self.factor):
             less = 0 != (l % self.factor) <= i
